@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009, 2011 Jerome Fisher
- * Copyright (C) 2012-2017 Jerome Fisher, Sergey V. Mikayev
+ * Copyright (C) 2012-2021 Jerome Fisher, Sergey V. Mikayev
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <cerrno>
 #include <cfloat>
 #include <climits>
+#include <clocale>
 #include <cmath>
 #include <cstdio>
 #include <cstdlib>
@@ -99,6 +100,7 @@ struct Options {
 	MT32Emu::AnalogOutputMode analogOutputMode;
 	MT32Emu::RendererType rendererType;
 	MT32Emu::SamplerateConversionQuality srcQuality;
+	int partialCount;
 	int rawChannelMap[8];
 	int rawChannelCount;
 
@@ -111,6 +113,8 @@ struct Options {
 	gboolean waitForReverb;
 	gboolean sendAllNotesOff;
 	gboolean niceAmpRamp;
+	gboolean nicePanning;
+	gboolean nicePartialMixing;
 };
 
 struct State {
@@ -139,6 +143,7 @@ static bool parseOptions(int argc, char *argv[], Options *options) {
 	gint analogOutputModeIx = 0;
 	gint rendererTypeIx = 0;
 	gint srcQualityIx = 2;
+	gint partialCount = MT32Emu::DEFAULT_MAX_PARTIALS;
 	gint outputSampleFormat = OUTPUT_SAMPLE_FORMAT_SINT16;
 	gint bufferFrameCount = DEFAULT_BUFFER_SIZE;
 	gint renderMinFrames = 0;
@@ -167,6 +172,8 @@ static bool parseOptions(int argc, char *argv[], Options *options) {
 	options->waitForReverb = true;
 	options->sendAllNotesOff = true;
 	options->niceAmpRamp = true;
+	options->nicePanning = false;
+	options->nicePartialMixing = false;
 	// FIXME: Perhaps there's a nicer way to represent long argument descriptions...
 	GOptionEntry entries[] = {
 		{"output", 'o', 0, G_OPTION_ARG_FILENAME, &options->outputFilename, "Output file (default: last source file name with \".wav\" appended)", "<filename>"},
@@ -186,6 +193,9 @@ static bool parseOptions(int argc, char *argv[], Options *options) {
 		 "                 2: GOOD\n"
 		 "                 3: BEST", "<src_quality>"},
 
+		{"max-partials", 'x', 0, G_OPTION_ARG_INT, &partialCount, "The maximum number of partials playing simultaneously.\n"
+		 "                (minimum: 8, default: 32)\n", "<max-partials>"},
+
 		{"analog-output-mode", 'a', 0, G_OPTION_ARG_INT, &analogOutputModeIx, "Analogue low-pass filter emulation mode (default: 0)\n"
 		 "                Ignored if -w is used (in which case 0/DISABLED is always used)\n"
 		 "                 0: DISABLED\n"
@@ -229,6 +239,11 @@ static bool parseOptions(int argc, char *argv[], Options *options) {
 		 "                WARNING: Sound can theoretically continue forever if not limited by other options.", NULL},
 		{"no-nice-amp-ramp", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &options->niceAmpRamp, "Emulate amplitude ramp accurately.\n"
 		 "                Quick changes of volume or expression on a MIDI channel may result in amp jumps which are avoided by default.", NULL},
+		{"nice-panning", 0, 0, G_OPTION_ARG_NONE, &options->nicePanning, "Enlarges the pan setting accuracy to 4 bits providing for smoother panning.\n"
+		 "                By default, we accurately emulate the LA32 chip that only supports the pan setting accuracy of 3 bits.", NULL},
+		{"nice-partial-mixing", 0, 0, G_OPTION_ARG_NONE, &options->nicePartialMixing, "Disables occasional counter-phase mixing of partials that the LA32 chip exhibits.\n"
+		 "                Timbres with closely souding partials may sound quite differently, or even cancel out if mixed counter-phase.\n"
+		 "                Enabling this option makes the behaviour more predictable.", NULL},
 
 		{"s", 's', 0, G_OPTION_ARG_FILENAME, &deprecatedSysexFile, "[DEPRECATED] Play this SMF or sysex file before any other. DEPRECATED: Instead just specify the file first in the file list.", "<midi_file>"},
 		{G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &options->inputFilenames, NULL, "<midi_file> [midi_file...]"},
@@ -267,6 +282,7 @@ static bool parseOptions(int argc, char *argv[], Options *options) {
 	} else {
 		options->bufferFrameCount = bufferFrameCount;
 	}
+	options->partialCount = partialCount < 8 ? 8 : partialCount;
 	options->renderMaxFrames = renderMaxFrames < 0 ? INT_MAX : renderMaxFrames;
 	options->renderMinFrames = renderMinFrames < 0 ? 0 : renderMinFrames;
 	if (options->renderMinFrames > options->renderMaxFrames) {
@@ -629,6 +645,7 @@ static void playSMF(smf_t *smf, const Options &options, State &state) {
 	int unterminatedSysexLen = 0;
 	unsigned char *unterminatedSysex = NULL;
 	unsigned long renderedFrames = 0;
+	smf_rewind(smf);
 	for (;;) {
 		smf_event_t *event = smf_get_next_event(smf);
 		unsigned long eventFrameIx;
@@ -717,9 +734,9 @@ static void playSMF(smf_t *smf, const Options &options, State &state) {
 	}
 	flushSilence(MIDI_ENDED, options, state);
 	if (options.sendAllNotesOff) {
-		for (unsigned char part = 0; part < 9; part++) {
-			state.service.playMsg(0x0040B0 & part); // Release sustain pedal
-			state.service.playMsg(0x007BB0 & part); // All notes off
+		for (unsigned char channel = 0; channel < 16; channel++) {
+			state.service.playMsg(0x0040B0 | channel); // Release sustain pedal
+			state.service.playMsg(0x007BB0 | channel); // All notes off
 		}
 	}
 	if (state.lastInputFile && options.renderMinFrames > state.renderedFrames) {
@@ -782,10 +799,11 @@ static bool playFile(const gchar *inputFilename, const gchar *displayInputFilena
 int main(int argc, char *argv[]) {
 	Options options;
 	MT32Emu::Service service;
+	setlocale(LC_ALL, "");
 	printf("Munt MT32Emu MIDI to Wave Conversion Utility. Version %s\n", VERSION);
 	printf("  Copyright (C) 2009, 2011 Jerome Fisher <re_munt@kingguppy.com>\n");
-	printf("  Copyright (C) 2012-2017 Jerome Fisher, Sergey V. Mikayev\n");
-	printf("Using Munt MT32Emu Library Version %s, libsmf Version %s\n", service.getLibraryVersionString(), smf_get_version());
+	printf("  Copyright (C) 2012-2021 Jerome Fisher, Sergey V. Mikayev\n");
+	printf("Using Munt MT32Emu Library Version %s, libsmf Version %s (with modifications)\n", service.getLibraryVersionString(), smf_get_version());
 	if (!parseOptions(argc, argv, &options)) {
 		return -1;
 	}
@@ -813,29 +831,38 @@ int main(int argc, char *argv[]) {
 	gchar *baseDir = options.romDir;
 	if (baseDir == NULL)
 		baseDir = (gchar *)"";
-	gchar pathName[2048];
-	g_strlcpy(pathName, baseDir, 2048);
-	g_strlcat(pathName, "CM32L_CONTROL.ROM", 2048);
+	gchar pathNameUtf8[2048];
+	g_strlcpy(pathNameUtf8, baseDir, 2048);
+	g_strlcat(pathNameUtf8, "CM32L_CONTROL.ROM", 2048);
+	gchar *pathName = g_locale_from_utf8(pathNameUtf8, strlen(pathNameUtf8), NULL, NULL, NULL);
 	if (service.addROMFile(pathName) != MT32EMU_RC_ADDED_CONTROL_ROM) {
-		g_strlcpy(pathName, baseDir, 2048);
-		g_strlcat(pathName, "MT32_CONTROL.ROM", 2048);
+		g_free(pathName);
+		g_strlcpy(pathNameUtf8, baseDir, 2048);
+		g_strlcat(pathNameUtf8, "MT32_CONTROL.ROM", 2048);
+		pathName = g_locale_from_utf8(pathNameUtf8, strlen(pathNameUtf8), NULL, NULL, NULL);
 		if (service.addROMFile(pathName) != MT32EMU_RC_ADDED_CONTROL_ROM) {
 			fprintf(stderr, "Control ROM not found.\n");
 			return 1;
 		}
 	}
-	g_strlcpy(pathName, baseDir, 2048);
-	g_strlcat(pathName, "CM32L_PCM.ROM", 2048);
+	g_free(pathName);
+	g_strlcpy(pathNameUtf8, baseDir, 2048);
+	g_strlcat(pathNameUtf8, "CM32L_PCM.ROM", 2048);
+	pathName = g_locale_from_utf8(pathNameUtf8, strlen(pathNameUtf8), NULL, NULL, NULL);
 	if (service.addROMFile(pathName) != MT32EMU_RC_ADDED_PCM_ROM) {
-		g_strlcpy(pathName, baseDir, 2048);
-		g_strlcat(pathName, "MT32_PCM.ROM", 2048);
+		g_free(pathName);
+		g_strlcpy(pathNameUtf8, baseDir, 2048);
+		g_strlcat(pathNameUtf8, "MT32_PCM.ROM", 2048);
+		pathName = g_locale_from_utf8(pathNameUtf8, strlen(pathNameUtf8), NULL, NULL, NULL);
 		if (service.addROMFile(pathName) != MT32EMU_RC_ADDED_PCM_ROM) {
 			fprintf(stderr, "PCM ROM not found.\n");
 			return 1;
 		}
 	}
+	g_free(pathName);
 	service.setStereoOutputSampleRate(options.sampleRate);
 	service.setSamplerateConversionQuality(options.srcQuality);
+	service.setPartialCount(options.partialCount);
 	service.setAnalogOutputMode(options.analogOutputMode);
 	service.selectRendererType(options.rendererType);
 	if (service.openSynth() == MT32EMU_RC_OK) {
@@ -843,6 +870,12 @@ int main(int argc, char *argv[]) {
 		if (!options.niceAmpRamp) {
 			service.setNiceAmpRampEnabled(false);
 		}
+		if (options.nicePanning) {
+			service.setNicePanningEnabled(true);
+		}
+		if (options.nicePartialMixing) {
+			service.setNicePartialMixingEnabled(true);
+		}
 		options.sampleRate = service.getActualStereoOutputSamplerate();
 		printf("Using output sample rate %d Hz\n", options.sampleRate);
 
